package org.artifactory.projects;

import lombok.NonNull;
import org.apache.commons.lang3.StringUtils;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.config.CentralConfigDiffValidation;
import org.artifactory.config.CentralConfigKey;
import org.jfrog.common.config.diff.DataDiff;
import org.jfrog.common.config.diff.DiffUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;

/**
 * @author BarakH
 */
@Component
public class ProjectAdminValidation implements CentralConfigDiffValidation {
    private static final Logger log = LoggerFactory.getLogger(ProjectAdminValidation.class);

    private final AuthorizationService authService;

    @Autowired
    public ProjectAdminValidation(AuthorizationService authService){
        this.authService = authService;
    }

    private final Set<String> approvedConfigChangesForProjectAdmin = Set
            .of(CentralConfigKey.localRepositoriesMap.getKey(),
                    CentralConfigKey.remoteRepositoriesMap.getKey(),
                    CentralConfigKey.virtualRepositoriesMap.getKey(),
                    CentralConfigKey.distributionRepositoriesMap.getKey(),
                    CentralConfigKey.releaseBundlesRepositoriesMap.getKey(),
                    "repoLayouts");

    /**
     * Don't call it with admin users. Make your authorization check before you are calling it.
     */
    public void validate(List<DataDiff<?>> diffResult) throws IllegalArgumentException {
        log.debug("Checking if project admin is allowed to make those Artifactory configuration changes");
        if(authService.isAdmin()){
            return;
        }
        for (DataDiff<?> diff : diffResult) {
            String field = getFieldName(diff);
            if (!approvedConfigChangesForProjectAdmin.contains(field)) {
                throw new IllegalArgumentException(
                        "Project admin is not allowed to make those Artifactory configuration changes.");
            }
        }
    }

    @NonNull
    private String getFieldName(DataDiff<?> diff) {
        String field = diff.getFieldName();
        if (StringUtils.isEmpty(field)) {
            return StringUtils.EMPTY;
        }
        if (field.contains(DiffUtils.DELIMITER)) {
            field = field.substring(0, field.indexOf(DiffUtils.DELIMITER));
        }
        return field;
    }
}
