package org.artifactory.api.events;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Nullable;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * EventLogHintManager (ThreadLocal API) allows adding hints (string values) to all events (node_events) which
 * will be created in the current thread.
 * <p><br>
 * <b>Don't leave your trash for other threads!</b><br>
 * It's important to call remove() method just after add() like in every other ThreadLocal API
 * <p><br>
 * Hints are useful when extra data is needed when handing an event
 * <br>For each new use case of this API a new type should be added by the developer to EventHintType
 * <p><br>
 * E.G: When deleting an artifact we need to save it's property value as a hint to later retrieve it when
 * handling the event
 * because the original property was removed and cannot be accessed anymore
 * <p>
 * Usage:
 * <pre>
 *
 * try {
 *     EventLogHintManager.add(EventHintType.DebianIndexing, "deletedPropertyValue")
 *     //delete artifact logic here will create events stored in the DB
 *     ....
 * } finally {
 *     EventLogHintManager.remove(EventHintType.DebianIndexing)
 * }
 * </pre>
 * Later on when need to retrieve the data, it can be accessed from the event.
 * <pre>
 *
 * EventInfo myEvent = ...;
 * String value = myEvent.getHint(EventHintType.DebianIndexing);
 * </pre>
 *
 * @author avishayh
 */
@Slf4j
public class EventLogHintManager {

    private static final ThreadLocal<EventLogHintManager> current = new ThreadLocal<>();

    private final Map<EventHintType, String> hintsByType = new HashMap<>();

    @Nullable
    public static Set<String> getHints() {
        EventLogHintManager manager = current.get();
        if (manager == null) {
            return null;
        }
        return manager.hintsByType.entrySet()
                .stream()
                .map(entry -> entry.getKey().getPrefix() + entry.getValue())
                .collect(Collectors.toSet());
    }

    public static void put(EventHintType hintTypeType, String value) {
        String existingValue = getOrCreate().hintsByType.put(hintTypeType, value);
        if (existingValue != null) {
            log.warn("Found a value for EventHintType '{}' on this thread, make sure you didn't forget to call " +
                    "remove() method", hintTypeType);
        }
    }

    public static void remove(EventHintType hintType) {
        EventLogHintManager manager = current.get();
        if (manager == null) {
            log.debug("Trying to call remove without add first");
            return;
        }
        String removed = manager.hintsByType.remove(hintType);
        if (removed == null) {
            log.debug("Couldn't find anything to remove for EventHintType {}", hintType);
        }

        //cleaning up if it is the last hint
        if (manager.hintsByType.isEmpty()) {
            current.remove();
        }
    }

    private static EventLogHintManager getOrCreate() {
        EventLogHintManager manager = current.get();
        if (manager == null) {
            manager = new EventLogHintManager();
            current.set(manager);
        }
        return manager;
    }

    @Nullable
    public static String getValue(@Nullable Set<String> hints, @NonNull EventHintType hintType) {
        if (hints == null) {
            return null;
        }
        String prefix = hintType.getPrefix();
        return hints.stream()
                .filter(value -> value != null && value.startsWith(prefix))
                .map(value -> value.substring(prefix.length()))
                .findAny().orElse(null);
    }
}
