package org.artifactory.aql.result;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonToken;
import com.github.fge.jackson.JacksonUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.artifactory.aql.model.AqlDomainEnum;
import org.artifactory.aql.model.AqlPermissionProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.HttpURLConnection;

import static org.artifactory.repo.RepoPath.REMOTE_CACHE_SUFFIX;

/**
 * This is the streamer for remote results received from a smart remote repo with the sent AqlQuery
 * The readNext() will read the next json object and return it
 *
 * @author Tomer Slutsky
 */
public class AqlJsonRemoteStreamer extends AqlRestResult implements AqlJsonStreamer {
    private static final Logger log = LoggerFactory.getLogger(AqlJsonRemoteStreamer.class);

    private final CloseableHttpResponse response;
    private final String baseRepoName;
    private JsonParser parser;
    private boolean isFinished;

    public AqlJsonRemoteStreamer(CloseableHttpResponse response, AqlPermissionProvider permissionProvider,
            String baseRepoName) {
        super(permissionProvider);
        this.response = response;
        this.baseRepoName = baseRepoName;
        this.isFinished = response.getStatusLine().getStatusCode() != HttpURLConnection.HTTP_OK;
        if (!this.isFinished) {
            this.initJsonParser();
            this.skipResultWrapper();
        }
    }

    public CloseableHttpResponse getResponse() {
        return this.response;
    }

    public void initJsonParser() {
        try {
            log.trace("The status code for the remote response is: {}", this.response.getStatusLine().getStatusCode());
            this.parser = factory.createParser(this.response.getEntity().getContent());
        } catch (IOException e) {
            log.error("Failed to initiate Aql JsonParser: {}", e.getMessage());
        }
    }

    @Override
    public void close() throws IOException {
        if (this.parser != null) {
            this.parser.close();
        } else {
            this.response.close();
        }
    }

    @Override
    public Row readNext() {
        if (!this.isFinished) {
            Row currRow = null;
            try {
                log.trace("Reading Row for the remote streamer for the repo: {}", this.baseRepoName);
                currRow = this.readRow();
                while (currRow != null &&
                        !this.canRead(AqlDomainEnum.items, currRow.itemRepo + REMOTE_CACHE_SUFFIX, currRow.itemPath,
                                currRow.itemName)) {
                    log.trace("The previous Row had no permissions for the following cache: {}, reading new Row",
                            currRow.itemRepo + REMOTE_CACHE_SUFFIX);
                    if (this.parser.currentToken() == JsonToken.END_ARRAY) {
                        currRow = null;
                        break;
                    }
                    currRow = this.readRow();
                }
            } catch (IOException e) {
                log.error("Failed to read Aql result: {}", e.getMessage());
                log.debug("Failed to read Aql result due to: ", e);
            }

            if (currRow != null) {
                return currRow;
            }
            log.trace("Read final row from remote streamer for the repo: {}", this.baseRepoName);
            this.isFinished = true;
        }

        return null;
    }

    private Row readRow() throws IOException {
        Row currRow = null;
        if (this.parser.currentToken() == JsonToken.END_ARRAY) {
            return null;
        }

        if (this.parser.nextToken() != JsonToken.END_ARRAY) {
            currRow = JacksonUtils.getReader().readValue(this.parser, Row.class);
            if (currRow != null) {
                currRow.itemRepo = this.baseRepoName;
            }
        }

        return currRow;
    }

    private void skipResultWrapper() {
        JsonToken token;
        try {
            token = this.parser.nextToken();

            while (token != JsonToken.START_ARRAY) {
                token = this.parser.nextToken();
            }

            this.parser.nextToken();
        } catch (IOException e) {
            log.error("Failed to skip Aql result wrapper: {}", e.getMessage());
            log.debug("Failed to skip Aql result wrapper due to: ", e);
        }
    }

    @Override
    public byte[] read() {
        return null;
    }

    @Override
    public byte[] readCompact() {
        return null;
    }
}
